@@ -4,19 +4,20 @@
 # ------------------------------------------------------------------
 #  LaTeX-specific driver stuff
 #
-#  Copyright  1994-1996, Matt Welsh
-#  Copyright  1996, Cees de Groot
-#  Copyright  1999-2002, Taketoshi Sano
-#  Copyright  1999, Kazuyuki Okamoto (euc-jp support in sgml2txt, sgml2html, and sgml2latex)
-#  Copyright  1999, Tetsu ONO (euc-jp support in sgml2txt, sgml2html, and sgml2latex)
-#  Copyright  2000, Juan Jose Amor (Support for PDF files)
-#  Copyright  2006-2020, Agustin Martin
+#  Copyright (C) 1994-1996, Matt Welsh
+#  Copyright (C) 1996, Cees de Groot
+#  Copyright (C) 1999-2002, Taketoshi Sano
+#  Copyright (C) 1999, Kazuyuki Okamoto (euc-jp support in sgml2txt, sgml2html, and sgml2latex)
+#  Copyright (C) 1999, Tetsu ONO (euc-jp support in sgml2txt, sgml2html, and sgml2latex)
+#  Copyright (C) 2000, Juan Jose Amor (Support for PDF files)
+#  Copyright (C) 2006-2020, Agustin Martin
 # ------------------------------------------------------------------
 
 package LinuxDocTools::fmt_latex2e;
 use strict;
 
 use LinuxDocTools::Utils qw(
+  ldt_err_log
   ldt_log
   ldt_which
   );
@@ -66,7 +67,9 @@ $latex2e->{pagenumber}     = 1;
 $latex2e->{quick}          = 0;
 $latex2e->{bibtex}         = 0;
 $latex2e->{makeindex}      = 0;
+$latex2e->{makeindexopts}  = "";
 $latex2e->{latex}          = "unknown";
+$latex2e->{latexopts}      = "";
 $latex2e->{dvips}          = "unknown";
 $latex2e->{verbosity}      = -1;
 $Formats{$latex2e->{NAME}} = $latex2e;
@@ -251,10 +254,11 @@ sub latex2e_fine_tune_options {
   # Fine tune $latex2e->{latex} for defaults
   if ( $latex2e->{latex} eq "unknown" ) {
     if ($global->{language} eq "ja"){
-      #  for Japanese jlatex users
-      $latex2e->{latex} = "jlatex";
-      # for Japanese platex users
-      #	$latex2e->{latex} = "platex";
+      if ($global->{charset} eq "utf-8") {
+	$latex2e->{latex} = "uplatex";
+      } else {
+	$latex2e->{latex} = "platex";
+      }
     } elsif ($global->{language} eq "ko") {
       # for Korean users
       $latex2e->{latex} = "hlatexp";
@@ -266,10 +270,13 @@ sub latex2e_fine_tune_options {
       $latex2e->{latex} = "latex";
     }
   }
-  #  Fine tune $latex2e->{latex} for some special cases
+
+  #  Fine tune $latex2e->{latex} and $latex2e->{latexopts} for special cases
   if ($latex2e->{latex} eq "hlatexp") {
     # for pdf: how about status?(for hlatex and hlatexp)
     $latex2e->{latex} = "hlatex";       # run hlatex if hlatexp is used
+  } elsif  ($latex2e->{latex} eq "platex") {
+    $latex2e->{latexopts} = "--kanji=euc"
   } elsif ($latex2e->{output} eq "pdf" && $latex2e->{latex} eq "latex" ) {
     $latex2e->{latex} = "pdflatex";
   }
@@ -277,7 +284,17 @@ sub latex2e_fine_tune_options {
   # Fine tune $latex2e->{dvips} for defaults and special cases
   if ($latex2e->{dvips} eq "unknown") {
     if ($global->{language} eq "ja"){
-      if ( $latex2e->{latex} eq "jlatex" ){
+      if ( $latex2e->{latex} eq "platex"
+	   or
+	   $latex2e->{latex} eq "uplatex"){
+	$latex2e->{dvips}  = "dvipdfmx";
+	# This sets output to pdf, but since behavior is dvips like we
+	# need to cheat the program using "ps" not to break code
+	# behind. This shoult be fixed in a cleaner way.
+	if ( $latex2e->{output} eq "pdf" ) {
+	  $latex2e->{output} = "ps";
+	}
+      } elsif ( $latex2e->{latex} eq "jlatex" ){
 	$latex2e->{dvips} = "dvi2ps";
       } else {
 	$latex2e->{dvips} = "dvips";
@@ -287,32 +304,42 @@ sub latex2e_fine_tune_options {
     }
   }
 
-  # Fine tune some other corner cases
-  # We use jlatex for Japanese encoded (euc-jp) characters.
-  # pdf support for Japanese are not yet. use ps for the time being.
-  if ($global->{charset} eq "nippon") {
-    $latex2e->{output} = "ps" if ($latex2e->{output} eq "pdf");
-  }
-
-  # Fine tune japanese with utf-8. Must use uplatex and dvipdfmx
-  if ($global->{charset} eq "utf-8") {
-    if ($global->{language} eq "ja"){
-      print STDERR "  - Force using uplatex and dvipdfmx for japanese+utf-8.\n";
-      ldt_log "  - Force using uplate and dvipdfmx for japanese+utf-8.";
-      $latex2e->{latex}  = "uplatex";
-      $latex2e->{dvips}  = "dvipdfmx";
-      # This sets output to pdf, but since behavior is dvips like we
-      # need to cheat the program using "ps" not to break code
-      # behind. This shoult be fixed in a cleaner way.
-      if ( $latex2e->{output} = "pdf" ) {
-	$latex2e->{output} = "ps";
-      }
+  # Choose a sensible default if $latex2e->{bibtex} is set (to 1)
+  if ( $latex2e->{bibtex} ) {
+    if ( $latex2e->{latex} eq "platex" ){
+      $latex2e->{bibtex} = "pbibtex";
+    } elsif ( $latex2e->{latex} eq "uplatex" ) {
+      $latex2e->{bibtex} = "upbibtex";
+    } else {
+      $latex2e->{bibtex} = "bibtex";
     }
   }
 
+  # Choose a sensible default if $latex2e->{makeindex} is set (to 1)
+  if ( $latex2e->{makeindex} ) {
+    if ( $latex2e->{latex} eq "platex" ){
+      $latex2e->{makeindex}     = "mendex";
+      $latex2e->{makeindexopts} = "-E";
+    } elsif ( $latex2e->{latex} eq "uplatex" ) {
+      $latex2e->{makeindex}    = "mendex";
+      $latex2e->{makeindexopts} = "-U";
+    } else {
+      $latex2e->{makeindex} = "makeindex";
+    }
+  }
+
+  # Fine tune some other corner cases
+  # We use jlatex for Japanese encoded (euc-jp) characters.
+  # pdf support for Japanese are not yet. use ps for the time being.
+  ###  if ($global->{charset} eq "nippon") {
+  ###    $latex2e->{output} = "ps" if ($latex2e->{output} eq "pdf");
+  ###  }
+
   ldt_log "  ${msgheader} general info after fine tuning:";
   ldt_log "    format: $global->{format}, charset: $global->{charset}, lang: $global->{language}";
-  ldt_log "    latex: $latex2e->{latex}, dvips: $latex2e->{dvips}, output: $latex2e->{output}, verbosity: $latex2e->{verbosity}";
+  ldt_log "    latex: $latex2e->{latex}, latexopts: $latex2e->{latexopts}, dvips: $latex2e->{dvips}";
+  ldt_log "    bibtex: $latex2e->{bibtex}, makeindex: $latex2e->{makeindex}";
+  ldt_log "    output: $latex2e->{output}, verbosity: $latex2e->{verbosity}";
 
   ldt_log "  Check if \"$latex2e->{latex}\" is available. ldt_which will fail otherwise";
   ldt_log "    ldt_which($latex2e->{latex}): " . ldt_which($latex2e->{latex})
@@ -343,7 +370,7 @@ sub latex2e_clean_tmplatexdir {
       . " Please check dir contents. They are to be removed manually.\n";
   } else {
     # ldt_log calls useful on LinucDocTools::Utils::remove_tmpfiles failure
-    my @suffixes = qw(aux bbl blg dlog dvi lof log lot out pdf ps tex toc);
+    my @suffixes = qw(aux bbl blg dlog dvi idx ilg ind lof log lot out pdf ps tex toc);
     for my $suffix (@suffixes) {
       my $file = "$tmplatexnam.$suffix";
       if ( -e $file ) {
@@ -529,6 +556,8 @@ $latex2e->{postASP} = sub
 	  elsif (/^\@\@LANG_PACKAGE\@\@/) {
 	    if ( $global->{'language'} eq "zh_CN" ){
 	      s/\@\@LANG_PACKAGE\@\@/\\usepackage\[UTF8\]\{ctex\}/;
+	    } elsif ($latex2e->{latex} eq "platex") {
+	      s/\@\@LANG_PACKAGE\@\@/\\UseRawInputEncoding/;
 	    } elsif ( $use_babel ) {
 	      s/\@\@LANG_PACKAGE\@\@/\\usepackage\[$latex_language\]\{babel\}/;
 	    } else {
@@ -604,7 +633,7 @@ $latex2e->{postASP} = sub
   chdir ($tmplatexdir);
 
   # Prepare actual LaTeX run(s)
-  my ($latexcommand) = "$latex2e->{latex} '\\nonstopmode\\input{$filename.tex}'";
+  my ($latexcommand) = "$latex2e->{latex} $latex2e->{latexopts} '\\nonstopmode\\input{$filename.tex}'";
   ldt_log "  ${msgheader}:latexcommand\n    $latexcommand";
 
   # Control verbosity for the different LaTeX runs
@@ -612,6 +641,7 @@ $latex2e->{postASP} = sub
   my $latex2suppress  = ' >/dev/null';
   my $latex3suppress  = "";
   my $bibtexsuppress  = "";
+  my $makeindexsuppress  = "";
 
   if ( $latex2e->{verbosity} > 1  ) {
     $latex1suppress  = "";
@@ -619,6 +649,7 @@ $latex2e->{postASP} = sub
   } elsif ( $latex2e->{verbosity} == 0 ) {
     $latex3suppress = ' >/dev/null';
     $bibtexsuppress =  ' >/dev/null';
+    $makeindexsuppress =  ' >/dev/null';
   } elsif ( $latex2e->{quick} ) {
     $latex1suppress  = "";
   }
@@ -628,14 +659,36 @@ $latex2e->{postASP} = sub
     die "$msgheader: LaTeX first run problem. Aborting ...\n";
   }
 
+  # Run bibtex if --bibtex option is set.
   if ( $latex2e->{bibtex} ) {
-    system ( "bibtex $filename" . $bibtexsuppress ) == 0
-      or print STDERR "$msgheader: Problems when running BibTeX. Ignoring ...\n";
+    my $bibtex_command = "$latex2e->{bibtex} $filename";
+    ldt_log "  ${msgheader}:bibtex_command\n    $bibtex_command";
+    system ( $bibtex_command . $bibtexsuppress ) == 0
+      or print STDERR "$msgheader: Problems running \"$latex2e->{bibtex}\". Ignoring ...\n";
+  } else {
+    ldt_log "  ${msgheader}: bibtex disabled";
   }
 
+  # Second and third run
   unless ( $latex2e->{quick} ){
+    ldt_log "  ${msgheader}: Starting laTeX second run.";
     system ($latexcommand . $latex2suppress) == 0
       or die "$msgheader: LaTeX second run problem. Aborting ...\n";
+    if ( $latex2e->{makeindex} ) {
+      ldt_log "  ${msgheader}: Starting makeindex run.";
+      my $idx_filename = "$filename.idx";
+      my $makeindex_command = "$latex2e->{makeindex} $latex2e->{makeindexopts} $idx_filename";
+      ldt_log "  ${msgheader}:makeindex_command\n    $makeindex_command";
+      if ( -e "$idx_filename" ) {
+	system ( $makeindex_command . $makeindexsuppress ) == 0
+	  or print STDERR "$msgheader: Problems running  $latex2e->{makeindex}. Ignoring ...\n";
+      } else {
+	ldt_err_log "  ${msgheader}: \"$idx_filename\" not found. Skipping makeindex step.";
+      }
+    } else {
+      ldt_log "  ${msgheader}: makeindex disabled.";
+    }
+    ldt_log "  ${msgheader}: Starting LaTeX third run.";
     system ( $latexcommand . $latex3suppress) == 0
       or die "$msgheader: LaTeX third run problem. Aborting ...\n";
   }
